@extends('Admin.layout')
@section('header')
    <script src="/tiny/tinymce.min.js"></script>

    <style>
        .notebook-lists.warning-background{
            background-color: #fdf6ec;
            color: #b88230;
        }
        .notebook-lists.success-background{
            background-color: #f0f9eb;
            color: #529b2e;
        }
        .notebook-lists{
            margin: 0 5px 5px;
            box-sizing: border-box;
            border-radius: 10px;
            padding:10px;
            font-size: 14px;
            position: relative;
        }
        .notebook-lists:hover{
            cursor: pointer;
        }
        .notebook-lists>.title{
            font-weight: bold;
            font-size: 17px;
            line-height: 30px;
        }
        .notebook-lists>.content{
            color: #303133;
            line-height: 26px;
        }
        .notebook-lists>.time{
            color: #909399;
            font-size: 13px;
            display: flex;
            justify-content: space-between;
        }
        .notebook-lists.warning-active{
            background-color: #f8e3c5!important;
        }
        .notebook-lists.success-active{
            background-color: #d1edc4!important;
        }
        .add{
            font-weight: bold;
            font-size: 22px;
            line-height: 65px;
            position: relative;
            top: 5px;
            margin: 0 5px;
            color: #529b2e;
        }
        .add:hover{
            cursor: pointer;
            color: #129b00;
        }
        .list-tool{
            position: absolute;
            top: 8px;
            right: 10px;
            display: flex;
        }
        .list-tool-item{
            margin: 0 5px;
            font-size: 15px;
        }
        .list-tool-item:hover{
            cursor: pointer;
            font-size: 18px;
            font-weight: bold;
        }
        .delete{
            color: red;
        }
        .share{
            color: #67C23A;
        }
        .allow-user{
            color: #409EFF;
        }
        .mode{
            color: #E6A23C;
        }
        #fr-logo{ display: none; }
        .fr-popup.fr-active{ z-index: 5 !important; }
        .edit-title{
            border: none;
            border-bottom: 1px solid #C0C4CC;
            outline: none;
            width: 100%;
            font-size: 20px;
            height: 50px;
            font-weight: bold;
            margin-bottom: 5px;
            padding-left: 5px;
            color: #303133;
            box-sizing: border-box;
        }
        .searchInput{
            padding-right: 64px;
            height: 50px;
        }
        .tool{
            position: absolute;
            right: 0;
            top: 10px;
        }
        #app{
            padding: 0 10px 0 5px;
        }
        .content-shade{
            position: absolute;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            z-index: 99;
            background-color: #f8e3c535;
        }
        .content-shade .logo{
            font-size: 100px;
            text-align: center;
            color: #f8e3c585;
            line-height: 150px;
            position: absolute;
            top: 50%;
            margin-top: -75px;
            width: 100%;
            user-select: none;
        }
        .delete-user{
            color: red;
        }
        .delete-user:hover{
            cursor: pointer;
        }
        .readonly{
            position: absolute;
            top: 0;
            left: 0;
            width: 100%;
            height: 110px;
            z-index: 99;
            background-color: #f8e3c535;
            color: #f89898;
            font-size: 30px;
            line-height: 110px;
            text-align: center;
        }
    </style>
@endsection
@section('main')
    <el-row>
        <el-col :span="6">
            <div style="margin: 0 5px 0;padding:0 5px;box-sizing: border-box;position:relative;">
                <input v-model="searchText" class="edit-title searchInput" @change="searchNotebook" placeholder="搜索">
                <div class="tool">
                    <el-icon class="add" @click="searchNotebook"><component :is="searchIcon"></component></el-icon>
                    <el-icon class="add" @click="create"><plus/></el-icon>
                </div>
            </div>
            <div v-infinite-scroll="reachBottom" :infinite-scroll-disabled="reachBottomDisabled" style="overflow: auto;height: calc(100vh - 80px)">
                <div @click="notebookToggle(index)" v-for="(item, index) in notebook" class="notebook-lists" :class="{
                'success-background':index % 2,
                'warning-background':!(index % 2),
                'warning-active':index === active && !(index % 2),
                'success-active':index === active && index % 2
            }" >
                    <div class="title">@{{item.title}}</div>
                    <div class="content">@{{item.content.replace(/<\/?.+?>/g,"").replace(/ /g,"").substring(0, 30)}}...</div>
                    <div class="time">创建时间：@{{item.create_time}}</div>
                    <div class="time"><div>创建人：@{{ item.create_name }}</div> <div>最后更新： @{{ item.latest_name }}</div></div>
                    <div class="list-tool">
                        <div v-if="item.user_mode === 'super'" class="list-tool-item delete" ><el-icon @click.stop="deleteaa(item, index)"><delete/></el-icon></div>
                        <div class="list-tool-item share" ><el-icon @click.stop="share(item, index)"><Share/></el-icon></div>
                        <div v-if="item.user_mode === 'super'" class="list-tool-item allow-user" ><el-icon @click.stop="allowUser(item, index)"><Avatar/></el-icon></div>
                        <div v-if="item.user_mode === 'super'" class="list-tool-item mode" ><el-icon @click.stop="mode(item, index)"><Operation/></el-icon></div>
                    </div>
                </div>
                <el-divider v-if="notNotebook">
                    没有更多笔记了
                </el-divider>
            </div>

        </el-col>
        <el-col :span="18" style="position:relative;">
            <div>
                <input v-model="currentNotebook.title" @change="titleChange" class="edit-title" placeholder="新的笔记">
            </div>
            <div id="tiny"></div>
            <div class="content-shade" v-if="!currentNotebook.title">
                <div class="logo">notebook</div>
            </div>
            <div class="readonly" v-if="currentNotebook.title && currentNotebook.user_mode === 'read'"> 只读模式，任何修改都无效 </div>
        </el-col>
    </el-row>

    {{--  分享  --}}
    <el-dialog v-model="shareVisible" title="分享地址">
        <el-link :underline="false" target="_blank" :href="shareUrl" type="primary">@{{ shareUrl }}</el-link>
        <a href="" target="_blank"></a>
    </el-dialog>

    {{--  模式  --}}
    <el-dialog v-model="modeVisible" title="调整模式">
        <el-radio-group v-model="updateNotebook.mode">
            <el-radio v-for="item in modeData" :label="item.value">@{{ item.label }}</el-radio>
        </el-radio-group>
        <template #footer>
          <span class="dialog-footer">
            <el-button @click="modeVisible = false">取消</el-button>
            <el-button type="primary" @click="mode()">
              确认修改
            </el-button>
          </span>
        </template>
    </el-dialog>
    {{--  协作用户  --}}
    <el-dialog v-model="allowVisible" width="500px" title="协作人员">
        <div style="display:flex;">
            <el-select style="flex: 1" v-model="waitAddAllowUser" :remote-method="userSearch" remote filterable multiple clearable placeholder="请选择">
                <el-option
                        v-for="item in adminDataOptions"
                        :key="item.value"
                        :label="item.label"
                        :value="item.value"
                />
            </el-select>

            <el-button style="margin-left: 10px" @click="addAllowUser">添加用户</el-button>
        </div>
        <div style="margin: 10px 0">
            <el-table :data="updateNotebook.user" style="width: 100%">
                <el-table-column prop="user" label="用户" width="180" >
                    <template #default="scope">
                        <span v-if="scope.row.user">@{{ scope.row.user }}</span>
                        <span v-else>用户名</span>
                    </template>
                </el-table-column>
                <el-table-column prop="mode" label="权限">
                    <template #default="scope">
                        <el-radio-group v-model="scope.row.mode" class="ml-4" @change="modeChange(scope.row)">
                            @foreach(\Plugins\Notebook\Enums\NotebookUserEnumMode::mapping() as $mode)
                                <el-radio :label="{{ $mode['value'] }}" size="large">{{ $mode['label'] }}</el-radio>
                            @endforeach
                        </el-radio-group>
                    </template>
                </el-table-column>
                <el-table-column width="50">
                    <template #default="scope">
                        <el-icon class="delete-user" @click.stop="deleteUser(scope.row, scope.$index)"><delete/></el-icon>
                    </template>
                </el-table-column>
            </el-table>
        </div>
    </el-dialog>
@endsection

@section('footer')
    <script>
        VueInit.data.notebook = [];
        VueInit.data.active = 0;
        VueInit.data.searchText = "";
        VueInit.data.searchIcon = "Search";
        VueInit.data.editor = "";
        VueInit.data.currentNotebook = {title:''};
        VueInit.data.noteBookVersion = 0;
        VueInit.data.noteBookUpVersion = 0;
        VueInit.data.page = 0;
        VueInit.data.reachBottomDisabled = true;
        VueInit.data.notNotebook = false;
        VueInit.data.shareVisible = false;
        VueInit.data.modeVisible = false;
        VueInit.data.allowVisible = false;
        VueInit.data.modeData = {!! json_encode(\Plugins\Notebook\Enums\NotebookEnumMode::mapping()) !!};
        VueInit.data.adminData = {!! json_encode(\App\Model\Base\Administrators::getSelectData('name')) !!};
        VueInit.data.adminDataOptions = []
        VueInit.data.shareUrl = "";
        VueInit.data.updateNotebook = "";
        VueInit.data.allowUserData = [];
        VueInit.data.waitAddAllowUser = [];
        VueInit.data.tinyInit = {
            "selector": "#tiny",
            "promotion": false,
            "branding": false,
            "convert_urls":false,
            "plugins": "quickbars fullscreen lists advlist table preview quickbars image codesample code link media searchreplace",
            "language": "zh-Hans",
            "quickbars_selection_toolbar": "bold italic underline forecolor backcolor",
            "quickbars_insert_toolbar": false,
            "images_upload_url": "/admin/common/upload",
            "file_picker_types": "file image media",
            "toolbar": "undo redo bold italic underline forecolor backcolor strikethrough align lineheight styleselect formatselect fontselect fontsizeselect bullist numlist outdent indent blocks blockquote removeformat image link codesample subscript superscript code hr fullscreen",
            "file_picker_callback": (callback, value, meta) => {
                let input = document.createElement('input');
                input.setAttribute("type", "file")
                if (meta.filetype === "image") {
                    input.setAttribute("accept", "image/*")
                } else if (meta.filetype === "media") {
                    input.setAttribute("accept", "video/*")
                } else {
                    input.setAttribute("accept", "*")
                }
                input.addEventListener("change", (e) => {
                    let file = e.target.files[0];
                    let formData = new FormData();
                    formData.append("file", file)
                    let loading = VueApp.$notify({
                        "message": "文件上传中,请稍后...",
                        "duration": 0,
                        "type": "warning",
                        "showClose": false
                    });
                    // 请求开始
                    axios({
                        "headers": {
                            "Content-Type": "multipart/form-data"
                        },
                        "data": formData,
                        "method": "post",
                        "url": "/admin/common/upload"
                    }).then(({data}) => {
                        callback(data.data)
                    }).catch((error) => {
                        VueApp.$message({message: error.message, type: "error"})
                    }).finally(() => {
                        loading.close()
                    })
                })
                input.click()
            },
            "images_upload_handler": (blobInfo, progress) => {

                return new Promise((resolve, reject) => {
                    const xhr = new XMLHttpRequest();
                    xhr.withCredentials = false;
                    xhr.open('POST', '/admin/common/upload');

                    xhr.upload.onprogress = (e) => {
                        progress(e.loaded / e.total * 100);
                    };

                    xhr.onload = () => {
                        if (xhr.status === 403) {
                            reject({message: 'HTTP Error: ' + xhr.status, remove: true});
                            return;
                        }

                        if (xhr.status < 200 || xhr.status >= 300) {
                            reject('HTTP Error: ' + xhr.status);
                            return;
                        }

                        const json = JSON.parse(xhr.responseText);
                        if (json.code !== 200) {
                            reject('上传失败: ' + json.msg);
                            return;
                        }

                        console.log(json.data);
                        resolve(json.data);
                    };

                    xhr.onerror = () => {
                        reject('Image upload failed due to a XHR Transport error. Code: ' + xhr.status);
                    };

                    const formData = new FormData();
                    formData.append('file', blobInfo.blob(), blobInfo.filename());

                    xhr.send(formData);
                });
            }, "height": window.innerHeight - 60,
            setup: (editor) => {
                editor.on('change', () => {
                    VueApp.currentNotebook.content = editor.getContent();
                    VueApp.noteBookVersion++;
                });
                editor.on('init', () => {
                    editor.options.set('readonly', true);
                });
            }
        };

        VueInit.methods.searchNotebook = function () {
            this.page = 1;
            this.getList();
        };

        VueInit.methods.mode = function (item, index) {
            if (!item) {
                this.update(this.updateNotebook, (res) =>{
                    this.updateNotebook.update_time = this.toTime();
                });
                this.modeVisible = false;
                return;
            }
            this.modeVisible = true;
            this.updateNotebook = item;
        };

        VueInit.methods.allowUser = function (item, index) {
            this.allowVisible = true;
            this.updateNotebook = item;
        };

        VueInit.methods.deleteUser = function (row, index) {
            axios({
                url: "{{ \App\Util\Hy::route()->to([\Plugins\Notebook\Controller\Admin\NotebookUserController::class, 'destroy']) }}",
                method: 'post',
                data: {
                    ids: [row.id]
                },
            }).then(({ data }) => {
                if (data.code !== 200) {
                    this.$message.error(data.msg);
                }else{
                    this.$message.success('更新成功');
                    this.getNotebookUsers(this.updateNotebook.id, (data) => {
                        this.updateNotebook.user = data;
                    });
                }
            });
        };

        VueInit.methods.modeChange = function (row) {
            axios({
                url: "{{ \App\Util\Hy::route()->to([\Plugins\Notebook\Controller\Admin\NotebookUserController::class, 'update']) }}",
                method: 'post',
                data: row,
            }).then(({ data }) => {
                if (data.code !== 200) {
                    row.mode = row.mode === 1 ? 2 : 1;
                    this.$message.error(data.msg);
                }else{
                    this.$message.success('更新成功');
                }
            });
        };

        VueInit.methods.getNotebookUsers = function (notebook_id, success){
            axios({
                url: "{{ \App\Util\Hy::route()->to([\Plugins\Notebook\Controller\Admin\NotebookUserController::class, 'listsData']) }}",
                method: 'get',
                params: {
                    notebook_id:notebook_id
                },
            }).then(({ data }) => {
                if (data.code === 200) {
                    success && success(data.data)
                }
            });
        }

        VueInit.methods.addAllowUser = function () {
            if (this.waitAddAllowUser.length > 0) {
                let waitAddAllowUser = [];

                if (this.waitAddAllowUser.includes('*')) {
                    this.waitAddAllowUser = this.adminData.map(v => v.value);
                }

                this.waitAddAllowUser.map(v =>{
                    if (!this.updateNotebook.user.map(v1 => v1.administrators_id).includes(v) && v !== '*') {
                        waitAddAllowUser.push(v);
                    }
                });
                this.waitAddAllowUser = [];
                if (waitAddAllowUser.length > 0) {
                    this.addAllowUserRequest(waitAddAllowUser);
                }
            }
        };

        VueInit.methods.addAllowUserRequest = function (waitAddAllowUser){
            axios({
                url: 'add-user',
                method: 'post',
                data: {
                    notebook_id: this.updateNotebook.id,
                    user_ids: waitAddAllowUser
                }
            }).then(({data}) =>{
                if (data.code === 200) {
                    this.$message.success("设置成功");
                    this.getNotebookUsers(this.updateNotebook.id, (data) => {
                        this.updateNotebook.user = data;
                    })
                }else{
                    this.$message.error("设置失败");
                }
            })
        }

        VueInit.methods.delAllowUser = function (index) {
            this.allowUserData.splice(index, 1);
        };

        VueInit.methods.userSearch = function (query){
            let list = [];
            if (query) {
                this.adminData.map(v => {
                    console.log(v);
                    console.log(v.label.includes(query));

                    v.label.indexOf(query) >= 0 && list.push(v);
                });
            }

            this.adminDataOptions = list;
        }

        VueInit.methods.titleChange = function (){
            this.noteBookVersion++;
        }

        VueInit.methods.editorInit = function (){
            tinymce.init(this.tinyInit);
        }
        VueInit.methods.reachBottom = function () {
            if(this.notNotebook){
                return;
            }
            this.page++;
            this.getList();
        };


        VueInit.event.addHandler('mounted:editorInit');
        VueInit.event.addHandler('created:aaaaaa', function (){
            if (this.notebook.length >= 10) {
                this.reachBottomDisabled = false;
            }else{
                this.getList();
            }
        });

        VueInit.methods.notebookToggle = function (index){
            this.active = index;
            this.currentNotebook.update_time = this.toTime();
            this.update();
            this.currentNotebook = this.notebook[index];
            this.noteBookVersion = 0;
            this.noteBookUpVersion = 0;

            tinymce.get('tiny').resetContent(this.currentNotebook.content);
            tinymce.get('tiny').mode.set(this.currentNotebook.user_mode === 'read' ? "readonly" : 'design')

            this.getLatest();
        }

        VueInit.methods.deleteaa = function (item, index){
            this.$confirm("确认删除笔记吗？", {
                confirm:"确认"
            }).then(() =>{
                this.notebook.splice(index, 1);
                if (index === this.active) {
                    this.currentNotebook = this.notebook[index] ? this.notebook[index] : {title: ''};
                }
                axios({
                    url:"destroy",
                    method:'post',
                    data:{
                        ids:[item.id]
                    }
                }).then(({data:res}) => {
                    if(res.code === 200){
                        this.$message.success("删除成功");
                    }else{
                        this.$message.error(res.msg);
                    }
                });
            })
        }

        VueInit.methods.create = function (){
            this.notebook.unshift({
                title:"新的笔记",
                content:''
            });
            this.currentNotebook = this.notebook[0];
            axios({
                url: "create",
                method: 'post',
                data: this.currentNotebook
            }).then(({ data:res }) => {
                if (res.code === 200) {
                    for (let key in res.data) {
                        this.currentNotebook[key] = res.data[key] ;
                    }
                }else{
                    this.$message.error('创建失败！');
                }
            });
        }

        VueInit.methods.update = function (data, call){
            let upVersion;
            if (!data) {
                upVersion = this.noteBookVersion;
                if(this.noteBookUpVersion === upVersion){
                    call && call();
                    return;
                }
            }

            data = data ? data : this.currentNotebook;
            if (data.user_mode === 'read') {
                call && call();
                return;
            }

            axios({
                url: "update",
                method: 'post',
                data: data
            }).then(({ data:res }) => {
                if(res.code !== 200){
                    this.$message.warning(res.msg);
                }else{
                    this.$notify.success("文档已同步");
                }
                call && call(res);
            });

            if (upVersion){
                this.noteBookUpVersion = upVersion;
            }
        }
        VueInit.methods.toTime = function () {
            var date = new Date();//时间戳为10位需*1000，时间戳为13位的话不需乘1000
            var Y = date.getFullYear() + '-';
            var M = (date.getMonth()+1 < 10 ? '0'+(date.getMonth()+1):date.getMonth()+1) + '-';
            var D = (date.getDate()< 10 ? '0'+date.getDate():date.getDate())+ ' ';
            var h = (date.getHours() < 10 ? '0'+date.getHours():date.getHours())+ ':';
            var m = (date.getMinutes() < 10 ? '0'+date.getMinutes():date.getMinutes()) + ':';
            var s = date.getSeconds() < 10 ? '0'+date.getSeconds():date.getSeconds();
            return Y+M+D+h+m+s;
        }

        VueInit.event.addHandler('created:updateRegularly', function (){
            let t = setTimeout(() => {
                this.update(null, (res) => {
                    if(res && res.code === 200){
                        this.currentNotebook.update_time = res.data.update_time;
                    }
                    this.$nextTick(() => {
                        this.getLatest();
                    })
                    clearTimeout(t);
                    this.updateRegularly();
                });
            }, 3000);
        });

        VueInit.methods.getLatest = function (){
            if (!this.currentNotebook.id) {
                return;
            }
            axios({
                url: "get-latest",
                params: {
                    id:this.currentNotebook.id,
                    update_time:this.currentNotebook.update_time,
                }
            }).then(({ data:res }) => {
                if (!res.data) {
                    return;
                }
                this.$notify.success("已获取获取最新笔记");
                this.currentNotebook.latest_name = res.data.latest_name;
                this.currentNotebook.content = res.data.content;
                this.currentNotebook.title = res.data.title;
                this.currentNotebook.update_time = res.data.update_time;
                tinymce.get('tiny').setContent(res.data.content);
            });
        }

        VueInit.methods.getList = function (){
            this.reachBottomDisabled = true;
            axios({
                url:"lists-data",
                params:{
                    search:{
                        searchType: {title:'like'},
                        search: {title:this.searchText},
                    },
                    page:this.page,
                    limit:10
                }
            }).then(({ data:res }) => {
                this.reachBottomDisabled = false;
                if (this.page === 1){
                    this.notebook = res.data.data;
                }else{
                    this.notebook.push(...res.data.data);
                }
                if (res.data.data < 10){
                    this.notNotebook = true;
                }
            })
        }

        VueInit.methods.share = function (item,index){
            this.shareVisible = true;
            this.shareUrl = `{{ \App\Util\Hy::route()->to([\Plugins\Notebook\Controller\Admin\SharePreviewController::class, 'index']) }}?key=${item.share_key}`;
        }

    </script>
@endsection